# -*- coding: utf-8; -*-
################################################################################
#
#  Sideshow -- Case/Special Order Tracker
#  Copyright © 2024-2025 Lance Edgar
#
#  This file is part of Sideshow.
#
#  Sideshow is free software: you can redistribute it and/or modify it
#  under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Sideshow is distributed in the hope that it will be useful, but
#  WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with Sideshow.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
Sideshow Order Handler
"""

from wuttjamaican.app import GenericHandler


class OrderHandler(GenericHandler):
    """
    Base class and default implementation for the :term:`order
    handler`.

    This is responsible for business logic involving customer orders
    after they have been first created.  (The :term:`new order batch`
    handler is responsible for creation logic.)
    """

    def get_order_qty_uom_text(self, order_qty, order_uom, case_size=None, html=False):
        """
        Return the display text for a given order quantity.

        Default logic will return something like ``"3 Cases (x 6 = 18
        Units)"``.

        :param order_qty: Numeric quantity.

        :param order_uom: An order UOM constant; should be something
           from :data:`~sideshow.enum.ORDER_UOM`.

        :param case_size: Case size for the product, if known.

        :param html: Whether the return value should include any HTML.
           If false (the default), it will be plain text only.  If
           true, will replace the ``x`` character with ``&times;``.

        :returns: Display text.
        """
        enum = self.app.enum

        if order_uom == enum.ORDER_UOM_CASE:
            if case_size is None:
                case_qty = unit_qty = '??'
            else:
                case_qty = self.app.render_quantity(case_size)
                unit_qty = self.app.render_quantity(order_qty * case_size)
            CS = enum.ORDER_UOM[enum.ORDER_UOM_CASE]
            EA = enum.ORDER_UOM[enum.ORDER_UOM_UNIT]
            order_qty = self.app.render_quantity(order_qty)
            times = '&times;' if html else 'x'
            return (f"{order_qty} {CS} ({times} {case_qty} = {unit_qty} {EA})")

        # units
        unit_qty = self.app.render_quantity(order_qty)
        EA = enum.ORDER_UOM[enum.ORDER_UOM_UNIT]
        return f"{unit_qty} {EA}"

    def item_status_to_variant(self, status_code):
        """
        Return a Buefy style variant for the given status code.

        Default logic will return ``None`` for "normal" item status,
        but may return ``'warning'`` for some (e.g. canceled).

        :param status_code: The status code for an order item.

        :returns: Style variant string (e.g. ``'warning'``) or
           ``None``.
        """
        enum = self.app.enum
        if status_code in (enum.ORDER_ITEM_STATUS_CANCELED,
                           enum.ORDER_ITEM_STATUS_REFUND_PENDING,
                           enum.ORDER_ITEM_STATUS_REFUNDED,
                           enum.ORDER_ITEM_STATUS_RESTOCKED,
                           enum.ORDER_ITEM_STATUS_EXPIRED,
                           enum.ORDER_ITEM_STATUS_INACTIVE):
            return 'warning'
